@@ -24,6 +24,7 @@ try:                    # Python 3
     int_types = int
 except ImportError:     # Python 2
     import ConfigParser as configparser
+    from itertools import ifilter as filter
     from threading import currentThread as current_thread
     from xmlrpclib import Fault, ServerProxy
     int_types = int, long
@@ -58,8 +59,13 @@ except ImportError:     # Python 2.5
             node_or_string = node_or_string.body
         return _convert(node_or_string)
 
+    def next(iterator, default):
+        for item in iterator:
+            return item
+        return default
 
-__version__ = '1.4.4'
+
+__version__ = '1.4.5'
 __all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service',
            'format_exception', 'read_config', 'start_openerp_services']
 
@@ -129,8 +135,7 @@ _methods_6_1 = {
     'report': ['render_report'],
 }
 # Hidden methods:
-#  - 'common': ['get_available_updates', 'get_migration_scripts',
-#               'set_loglevel']
+#  - common: get_available_updates, get_migration_scripts, set_loglevel
 _cause_message = ("\nThe above exception was the direct cause "
                   "of the following exception:\n\n")
 
@@ -159,7 +164,7 @@ def lowercase(s, _sub=re.compile('[A-Z]').sub,
         return _cache[s]
     except KeyError:
         _cache[s] = s = _sub(_repl, s).lstrip('.')
-        return s
+    return s
 
 
 def format_exception(exc_type, exc, tb, limit=None, chain=True,
@@ -226,7 +231,7 @@ def start_openerp_services(options=None):
     global get_pool
     if not openerp.osv.osv.service:
         os.environ['TZ'] = 'UTC'
-        options = options and options.split() or []
+        options = options.split() if options else []
         openerp.tools.config.parse_config(options)
         if openerp.release.version_info < (7,):
             openerp.netsvc.init_logger()
@@ -331,10 +336,10 @@ class Service(object):
         if name not in self._methods:
             raise AttributeError("'Service' object has no attribute %r" % name)
         if self._verbose:
-            def sanitize(args, _pos=(self._endpoint == 'db') and 999 or 2):
-                if len(args) > _pos:
+            def sanitize(args):
+                if self._endpoint != 'db' and len(args) > 2:
                     args = list(args)
-                    args[_pos] = '*'
+                    args[2] = '*'
                 return args
             maxcol = MAXCOL[min(len(MAXCOL), self._verbose) - 1]
 
@@ -842,7 +847,7 @@ class Client(object):
         """
         domain = [('name', 'like', name)]
         if installed is not None:
-            op = installed and 'not in' or 'in'
+            op = 'not in' if installed else 'in'
             domain.append(('state', op, ['uninstalled', 'uninstallable']))
         mods = self.read('ir.module.module', domain, 'name state')
         if mods:
@@ -973,17 +978,25 @@ class Model(object):
         """Return a single :class:`Record`.
 
         The argument `domain` accepts a single integer ``id`` or a
-        search domain.  The return value is a :class:`Record` or None.
-        If multiple records are found, a ``ValueError`` is raised.
+        search domain, or an ``xml_id``.  The return value is a
+        :class:`Record` or None.  If multiple records are found,
+        a ``ValueError`` is raised.
         """
-        if isinstance(domain, int_types):
+        if isinstance(domain, int_types):   # a single id
             return Record(self, domain, context=context)
-        assert issearchdomain(domain)
-        params = searchargs((domain,), context=context)
-        ids = self._execute('search', *params)
+        if isinstance(domain, basestring):  # lookup the xml_id
+            module, name = domain.split('.')
+            data = self.client.model('ir.model.data').read(
+                [('module', '=', module), ('name', '=', name)], 'model res_id')
+            assert not data or data[0]['model'] == self._name
+            ids = [res['res_id'] for res in data]
+        else:                               # a search domain
+            assert issearchdomain(domain)
+            params = searchargs((domain,), context=context)
+            ids = self._execute('search', *params)
         if len(ids) > 1:
             raise ValueError('domain matches too many records (%d)' % len(ids))
-        return ids and Record(self, ids[0], context=context) or None
+        return Record(self, ids[0], context=context) if ids else None
 
     def create(self, values, context=None):
         """Create a :class:`Record`.
@@ -1074,7 +1087,6 @@ class RecordList(object):
                 _ids.append(id_)
         # Bypass the __setattr__ method
         self.__dict__.update({
-            # 'client': res_model.client,
             'id': _ids,
             '_model_name': res_model._name,
             '_model': res_model,
@@ -1091,8 +1103,8 @@ class RecordList(object):
         return "<RecordList '%s,%s'>" % (self._model_name, ids)
 
     def __dir__(self):
-        return ['__getitem__', 'read', 'write', 'unlink',
-                'id', '_context', '_idnames', '_model', '_model_name']
+        return ['__getitem__', 'read', 'write', 'unlink', '_context',
+                '_idnames', '_model', '_model_name'] + self._model._keys
 
     def __len__(self):
         return len(self.id)
@@ -1106,9 +1118,9 @@ class RecordList(object):
         if self.id:
             values = client.read(self._model_name, self.id,
                                  fields, order=True, context=context)
-            if values and isinstance(values[0], dict):
+            if isinstance(next(filter(None, values), None), dict):
                 browse_values = self._model._browse_values
-                return [browse_values(v) for v in values]
+                return [v and browse_values(v) for v in values]
         else:
             values = []
 
@@ -1155,7 +1167,7 @@ class RecordList(object):
         idname = self._idnames[key]
         if idname is False:
             return False
-        cls = isinstance(key, slice) and RecordList or Record
+        cls = RecordList if isinstance(key, slice) else Record
         return cls(self._model, idname, context=self._context)
 
     def __getattr__(self, attr):
@@ -1267,7 +1279,7 @@ class Record(object):
         See :meth:`Client.perm_read` for details.
         """
         rv = self._execute('perm_read', [self.id], context=context)
-        return rv and rv[0] or None
+        return rv[0] if rv else None
 
     def write(self, values, context=None):
         """Write the `values` in the :class:`Record`."""
@@ -1308,8 +1320,8 @@ class Record(object):
 
     def __dir__(self):
         return ['read', 'write', 'copy', 'unlink', '_send', 'refresh',
-                'id', '_context', '_model', '_model_name',
-                '_name', '_keys', '_fields'] + self._model._keys
+                '_context', '_model', '_model_name', '_name',
+                '_keys', '_fields'] + self._model._keys
 
     def __getattr__(self, attr):
         context = self._context
@@ -1372,17 +1384,17 @@ def _interact(use_pprint=True, usage=USAGE):
     try:
         import readline as rl
         import rlcompleter
-    except ImportError:
+        rl.parse_and_bind('tab: complete')
+        # IOError if file missing, or broken Apple readline
+        rl.read_history_file(HIST_FILE)
+    except Exception:
         pass
     else:
         import atexit
-        rl.parse_and_bind('tab: complete')
-        if os.path.exists(HIST_FILE):
-            rl.read_history_file(HIST_FILE)
-            if rl.get_history_length() < 0:
-                rl.set_history_length(int(os.environ.get('HISTSIZE', 500)))
-            # better append instead of replace?
-            atexit.register(rl.write_history_file, HIST_FILE)
+        if rl.get_history_length() < 0:
+            rl.set_history_length(int(os.environ.get('HISTSIZE', 500)))
+        # better append instead of replace?
+        atexit.register(rl.write_history_file, HIST_FILE)
 
     class Console(code.InteractiveConsole):
         def runcode(self, code):
